# Copyright (c) 2006 Universit de Lige, Nicolas Mazziotta 
# $Id$
# This program is free software; you can redistribute it and/or
# modify it under the same terms as Perl itself.
#
# Extension de XML::LibXML permettant de traiter les
# analyses syntaxiques CCFM. 
#
#######################################################
#######################################################
package Ana;
#######################################################
#######################################################

use strict;		
use XML::LibXML;

#=====================================================#
sub  XML::LibXML::Document::AnaBuild
#=====================================================#
# Construit la liste de pointeurs et la place  l'endroit 
# voulu dans le document.
{
	my ($xml_doc, $target_nodes_xpath, $ana_node_xpath, $position, $mode, $formes) = @_;
	my $ana_node = XML::LibXML::Element->new("div");
	$ana_node->setAttribute("type", "analyse syntaxique");
	foreach ($xml_doc->findnodes($target_nodes_xpath)) {
		my $elt = XML::LibXML::Element->new("L");
		if ($formes) {
			my $text = $_->textContent;
			$text =~ s/\s+/ /g;
			$elt->setAttribute($formes, $text);
		}
		my $ref = $_->getAttribute("id");
		if ($ref) {
			$elt->setAttribute("target", $ref) 
		} else {
			die "Tous les lments qui sont cibls par l'analyse doivent tre pourvus d'identifiants!\n"
		};
		$ana_node->appendChild($elt);
	}
	$ana_node = &_work2share($ana_node) if $mode =~ /^s(h(a(r(e)?)?)?)?$/;
	$xml_doc->AnaAppend($ana_node, $ana_node_xpath, $position);
}

#=====================================================#
sub  XML::LibXML::Document::AnaAppend
#=====================================================#
# retourne le document modifi
# en collant le nom  l'endroit indiqu par le xpath
# et en ajustant sa position (before|after|firstChild|lastChild)
{
	my ($xml_doc, $ana_elt, $xpath_locator, $position) = @_;
	$position = "lastChild" unless $position;
	foreach ($xml_doc->findnodes($xpath_locator)) {
		if ($position eq "firstChild") {
			$_->insertBefore($ana_elt, $_->firstChild) ;
		} elsif ($position eq "before") {
			$_->insertBefore($ana_elt, $_) ;
		} elsif ($position eq "after") {
			$_->insertAfter($ana_elt, $_) ;
		}
		else {
			$_->appendChild($ana_elt);
		}
		last;
	}
	return $xml_doc;
}

#=====================================================#
sub  XML::LibXML::Document::AnaGet
#=====================================================#
# Trouve l'analyse et la coupe de l'arbre
{
	my ($xml_doc, $xpath_locator) = @_;
	foreach my $ana ($xml_doc->findnodes($xpath_locator)) {
		return $ana->parentNode->removeChild($ana);
	}
}

#=====================================================#
sub  XML::LibXML::Element::AnaConvert
#=====================================================#
# Convertit l'analyse
{
	my ($base_ana, $direction) = @_;
	if (!$direction) {
		my $ana_name = $base_ana->nodeName();
		if ($ana_name eq "anaGrp") { $direction = "work" } 
		else { $direction = "share" }
	} elsif ($direction =~ /^n(o(n(e)?)?)?$/) {	return $base_ana }
	my $converted_ana;
	if ($direction =~ /^w(o(r(k)?)?)?$/) {
		$converted_ana = &_share2work($base_ana);
	} elsif ($direction =~ /^s(h(a(r(e)?)?)?)?$/) {
		$converted_ana = &_work2share($base_ana);
	}
	return $converted_ana;
}

#=====================================================#
sub  XML::LibXML::Element::AnaPurge
#=====================================================#
# dtruit certaons noeuds de l'analyse (utile pour 
# enlever les noeuds qui explicitent les pointeurs)
{
	my ($ana_node, $xpath) = @_;
	foreach ($ana_node->findnodes($xpath)) {
		if ($_->nodeType == 2) {
			$_->parentNode->removeAttribute($_->nodeName)
		} else {
			$_->parentNode->removeChild($_);
		}
	}
	return $ana_node;
}

#=====================================================#
sub  _share2work
#=====================================================#
# (priv) convertit du format de travail au format 
# de partage
{
	my ($share_ana) = @_;
	my $work_ana_node;
	my $work_ana_notes;
	my $type = $share_ana->getAttribute("type");
	foreach ($share_ana->findnodes("./anaChunk")) {
		$work_ana_node = $share_ana->removeChild($_);
		$work_ana_node->setAttribute("type", $type);
		$work_ana_node->setNodeName("div");
		last
	}
	foreach ($share_ana->findnodes("./anaNotes")) {
		$work_ana_notes = &__ana_struct_from_xml($_);
		last
	}
	
	foreach my $seg ($work_ana_node->findnodes(".//seg")) {
		my $ref = $seg->getAttribute("id");
		die "Tous les lments d'analyse doivent tre pourvus d'identifiants!\n" unless $ref;
		$seg->setNodeName("S");
		if ($ref and my $data = $work_ana_notes->{$ref}) {
			foreach (@$data) { $seg->setAttribute(%$_);	}
		}
		foreach ($seg->findnodes("./link")) {
			$_->setNodeName("L");
		}
	}
	
	
	return $work_ana_node;
}

#=====================================================#
sub  _work2share
#=====================================================#
# (priv) inverse de _share2work
{
	my ($work_ana_node) = @_;
	my $anaGrp_node = XML::LibXML::Element->new("anaGrp");

	my $type = $work_ana_node->getAttribute("type");
	$type = "" unless $type;
	$anaGrp_node->setAttribute("type", $type);

	# contiendra l'arbre final
	my $tree_node = $work_ana_node->cloneNode(1); 	
	$tree_node->removeAttribute("type");
	# contiendra les donnes d'annotation, puis sera srialis en XML
	my $annotations_struct = {};
 	
	$tree_node->setNodeName("anaChunk");
	
	foreach my $seg_node ($tree_node->findnodes(".//S")) {
		# on change le nom des noeuds
		$seg_node->setNodeName("seg");
		my $ref;
		# on extrait la valeur de chaque attribut
		foreach ($seg_node->findnodes("./attribute::*")) {
			my $name = $_->nodeName;
			my $value = $_->value;
			if ($name eq "id") {
				$ref = $value;
				next
			} 
			# on range tout dans le hashref qui structure les annotations
			push @{$annotations_struct->{$name}{$value}}, $ref;
			$seg_node->removeAttribute($name);
		}
		foreach ($seg_node->findnodes("./L")) {
			$_->setNodeName("link");
		}
	}
	foreach ($tree_node->findnodes("./L")) {
		$_->setNodeName("link");
	}
	
	my $anaChunk_node = $tree_node;
	$anaGrp_node->appendChild($anaChunk_node);
	
	my $anaNotes_node = &__ana_struct_2_xml($annotations_struct);
	$anaGrp_node->appendChild($anaNotes_node);

	return $anaGrp_node; 
}

#=====================================================#
sub  __ana_struct_from_xml
#=====================================================#
# (priv) Convertit les donnes de l'analyse (format de
# partage) en structure de donnes perl.
{
	my ($struct_node) = @_;
	my $struct = {};
	foreach my $tag_node ($struct_node->findnodes("./anaTag")) {
		my $tag = $tag_node->getAttribute("type");
		foreach my $value_node ($tag_node->findnodes("./anaValue")) {
			my $value = $value_node->getAttribute("type");
			foreach my $ref ($value_node->findnodes("./linkGrp/link/attribute::target")) {
				push @{$struct->{$ref->value}}, {$tag => $value};
			}
		}
	}
	return $struct;
}


#=====================================================#
sub  __ana_struct_2_xml
#=====================================================#
# (priv) inverse de __ana_struct_from_xml
{
	my ($struct) = @_;
	my $anaNotes_node = XML::LibXML::Element->new("anaNotes");
	foreach my $tag (sort keys %{$struct}) {
		my $anaTag_node = XML::LibXML::Element->new("anaTag");
		$anaTag_node->setAttribute("type", $tag);
		foreach my $value (sort keys %{$struct->{$tag}}) {
			my $anaValue_node = XML::LibXML::Element->new("anaValue");
			$anaValue_node->setAttribute("type", $value);
			my $linkGrp_node = XML::LibXML::Element->new("linkGrp");
			foreach my $ref (@{$struct->{$tag}{$value}}) {
				my $link_node = XML::LibXML::Element->new("link");
				$link_node->setAttribute("target", $ref);
				$linkGrp_node->appendChild($link_node);
			}
			$anaValue_node->appendChild($linkGrp_node);
			$anaTag_node->appendChild($anaValue_node);
		}
		$anaNotes_node->appendChild($anaTag_node);
	}
	return $anaNotes_node;
}

1;

# vim:ts=2 sw=2

